import adafruit_ds3231
import board
import busio
import displayio
import digitalio
import neopixel
import random
import rotaryio
import time
import TM1637


days = ("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")

i2c = busio.I2C(board.GP21, board.GP20)
rtc = adafruit_ds3231.DS3231(i2c)
t = rtc.datetime


num_leds = 24  # 24 LED NeoPixel ring
neopixel_pin = board.GP0
strip = neopixel.NeoPixel(neopixel_pin, num_leds)

brightness_high = 0.7  # Active brightness
brightness_low = 0.125  # Idle brightness
offset = 0  # Animation position

#GPIO SETUP
encoder = rotaryio.IncrementalEncoder(board.GP16, board.GP17)

btn_Select = digitalio.DigitalInOut(board.GP15)
btn_Select.direction = digitalio.Direction.INPUT
btn_Select.pull = digitalio.Pull.UP

btn_D = digitalio.DigitalInOut(board.GP19)
btn_D.direction = digitalio.Direction.INPUT
btn_D.pull = digitalio.Pull.UP

btn_T = digitalio.DigitalInOut(board.GP18)
btn_T.direction = digitalio.Direction.INPUT
btn_T.pull = digitalio.Pull.UP

btn_AlarmOff = digitalio.DigitalInOut(board.GP28)
btn_AlarmOff.direction = digitalio.Direction.INPUT
btn_AlarmOff.pull = digitalio.Pull.UP

btn_DIP1 = digitalio.DigitalInOut(board.GP2)
btn_DIP1.direction = digitalio.Direction.INPUT
btn_DIP1.pull = digitalio.Pull.UP

btn_DIP2 = digitalio.DigitalInOut(board.GP3)
btn_DIP2.direction = digitalio.Direction.INPUT
btn_DIP2.pull = digitalio.Pull.UP

btn_DIP3 = digitalio.DigitalInOut(board.GP4)
btn_DIP3.direction = digitalio.Direction.INPUT
btn_DIP3.pull = digitalio.Pull.UP

btn_DIP4 = digitalio.DigitalInOut(board.GP5)
btn_DIP4.direction = digitalio.Direction.INPUT
btn_DIP4.pull = digitalio.Pull.UP

btn_DIP5 = digitalio.DigitalInOut(board.GP6)
btn_DIP5.direction = digitalio.Direction.INPUT
btn_DIP5.pull = digitalio.Pull.UP

btn_Hour = digitalio.DigitalInOut(board.GP13)
btn_Hour.direction = digitalio.Direction.INPUT
btn_Hour.pull = digitalio.Pull.UP

btn_Minute = digitalio.DigitalInOut(board.GP14)
btn_Minute.direction = digitalio.Direction.INPUT
btn_Minute.pull = digitalio.Pull.UP

LED_Tower = digitalio.DigitalInOut(board.GP8)
LED_Tower.direction = digitalio.Direction.OUTPUT

Fan = digitalio.DigitalInOut(board.GP10)
Fan.direction = digitalio.Direction.OUTPUT

Buzzer = digitalio.DigitalInOut(board.GP22)
Buzzer.direction = digitalio.Direction.OUTPUT


position = 0
Alarm_H = 12
Alarm_M = 0
BTN_H_OK = 0
BTN_M_OK = 0
Alarm_ON = False
SoftWake = True
DIP5 = 0
Reset_H = 0
Reset_M = 0
Reset_Counter = 0


# =======       END INIT            =======
# =======       START SUBROUTINES   =======

def SetTheTime():       # SET UP THE CLOCK TIME IN RTC!
    global position
    global BTN_H_OK
    global BTN_M_OK
    yr = 2023
    mt = 10
    dy = 28
    sc = 00
    wd = 1
    hr = 0
    mn = 0
    while (btn_DIP2.value == False):
        display.brightness(7)
        display.numbers(hr, mn, True)
        time.sleep(0.05)
        Buzzer.value = True
        time.sleep(0.001)
        Buzzer.value = False
        Rotary_Switch()
        if(BTN_H_OK == 1):
            BTN_H_OK = 0
            hr = position
            #print ("Stunde", hr)
        if(BTN_M_OK == 1):
            BTN_M_OK = 0
            mn = position
            #print ("Minute", mn)

    #set_time = time.struct_time((2023, 10, 22, 16, 29, 20, 0, -1, -1)) # Year, Month, Date, Hour, Minutes, Seconds, Week Day
    #set_time = time.localtime()          # Get time from USB PC
    set_time = time.struct_time((yr, mt, dy, hr, mn, sc, wd, -1, -1)) # Year, Month, Date, Hour, Minutes, Seconds, Week Day
    rtc.datetime = set_time
    display.brightness(3)


def Rotary_Switch():
    global position
    global BTN_H_OK
    global BTN_M_OK
    last_position = None
    while btn_Hour.value == False or btn_Minute.value == False:
        display.number(position)
        position = encoder.position
        if(btn_Hour.value == False):
            BTN_H_OK = 1
        if(btn_Minute.value == False):
            BTN_M_OK = 1
        if(position > 23 and btn_Hour.value == False):
            position = 23
            last_position = 23
            encoder.position = 23
        if(position > 59 and btn_Minute.value == False):
            position = 59
            last_position = 59
            encoder.position = 59
        if (position < 0):
            position = 0
            last_position = 0
            encoder.position = 0
        if last_position is None or position != last_position:
            #print(position)
            Buzzer.value = True
            time.sleep(0.002)
            Buzzer.value = False
        last_position = position


def Pause():
    global Alarm_H
    global Alarm_M
    global btn_AlarmOff
    global Alarm_ON
    for i in range(50):
        if(btn_AlarmOff.value == False):
            AlarmCancel()
        if(btn_Select.value == False):
            strip.fill([0, 0, 0])
            strip.write()
            if (Alarm_H == 88):
                Buzzer.value = True
                time.sleep(0.1)
                Buzzer.value = False
                Alarm_H = 88
                Alarm_M = 88
            for x in range(8):
                if(btn_AlarmOff.value == False):
                    AlarmCancel()
                display.brightness(7)
                display.numbers(Alarm_H, Alarm_M, True)
                time.sleep(0.5)
                display.brightness(3)
                time.sleep(0.5)
        if (btn_Hour.value == False):
            SetAlarm_H()
        if btn_Minute.value == False:
            SetAlarm_M()
        if (btn_Minute.value == False):
            SetAlarm_M()
        t = random.randint(0,1)
        if (t == 1):
            t = random.randint(0,1)
            if (t == 1):
                t = random.randint(0,1)
        if (btn_D.value == True):
            #print("KIPPSCHALTER H")
            LED_Tower.value = 0
            display.brightness(0)
        else:
            LED_Tower.value = t
            #display.brightness(3)
        time.sleep(0.025)


def SetAlarm_H():
    global Alarm_H
    global Alarm_M
    global position
    LED7Seg_Ready()
    while btn_Hour.value == False:
        #print("STUNDENKNOPF PRESSED")
        Rotary_Switch()
        Alarm_H = position
        #print(Alarm_H)
        display.numbers(Alarm_H, Alarm_M, True)
        time.sleep(0.2)
    display.brightness(3)
    if (Alarm_M < 60):
        strip.fill([0, 0, 0])
        strip.write()


def SetAlarm_M():
    global Alarm_M
    global Alarm_H
    global position
    LED7Seg_Ready()
    while btn_Minute.value == False:
        #print("MINUTENKNOPF PRESSED")
        Rotary_Switch()
        Alarm_M = position
        #print(Alarm_M)
        display.numbers(Alarm_H, Alarm_M, True)
        time.sleep(0.2)
    display.brightness(3)
    if (Alarm_H < 60):
        strip.fill([0, 0, 0])
        strip.write()


def LED7Seg_Ready():
    display.brightness(0)
    for i in range(3):
        display.brightness(0)
        time.sleep(0.05)
        display.brightness(7)
        time.sleep(0.05)


def AlarmLight():
    #print("LEDRING")
    global Alarm_ON
    global btn_AlarmOff
    strip.brightness = 1
    for i in range(0, num_leds, 1):
        if(Alarm_ON == True):
            strip.fill([0, 0, 0])
            if(btn_AlarmOff.value == False):
                AlarmCancel()
        strip[(offset + i) % num_leds] = 0,0,255
        strip[(offset + (i+1)) % num_leds] = 0,0,255
        time.sleep(0.05)
        strip.fill([0, 0, 0])
    strip.write()  # and issue data to LED strip

def AlarmLight_One():
    R = random.randint(1,255)
    G = random.randint(1,255)
    B = random.randint(1,255)
    strip[(offset + random.randint(1,12)) % num_leds] = 0,0,R
    strip[(offset + random.randint(1,12)) % num_leds] = 0,G,0
    strip[(offset + random.randint(1,12)) % num_leds] = B,0,0
    strip.write()

def Check_Alarm():
    global Stunde
    global Minute
    global Alarm_H
    global Alarm_M
    global Fan
    global Buzzer
    global Alarm_ON
    global num_leds
    Alarm_ON = False
    SysMin = (Stunde * 60) + Minute
    AlarmMin = (Alarm_H *60) + Alarm_M
    #print ("IstMinuten    :", SysMin)
    #print ("Alarm Minuten :", AlarmMin)
    #print ("Alarm in ", AlarmMin - SysMin, "Minuten")
    if (SysMin > (AlarmMin - 11) and SysMin < (AlarmMin - 4) and SoftWake == True): # 10 min befor wakeup time LED Ring is on
        AlarmLight()
        Alarm_ON = True
    Fan.value = False
    if (SysMin > (AlarmMin - 5) and SysMin < AlarmMin and SoftWake == True): # 4 min befor wakeup time fan is on
        #print("Running Fan Now")
        strip.fill([0, 0, 0])
        strip[(offset + random.randint(1,12)) % num_leds] = 0,0,255
        Fan.value = True
        strip.brightness = brightness_high
        strip.fill([255, 255, 255])
        time.sleep(0.1)
        strip.fill([0, 0, 0])
        Alarm_ON = True
    if (SysMin == AlarmMin):                             #wake up time
        time.sleep(0.01)
        Buzzer.value = True
        time.sleep(0.01)
        Buzzer.value = False
        Alarm_ON = True
        AlarmLight_One()
    if (SysMin == AlarmMin + 1):                         #1 min past wake up time
        time.sleep(0.2)
        Buzzer.value = True
        time.sleep(0.2)
        Buzzer.value = False
        Alarm_ON = True
        AlarmLight_One()
    if (SysMin == AlarmMin + 2):                         #2 min past wake up time
        time.sleep(0.3)
        Buzzer.value = True
        time.sleep(0.3)
        Buzzer.value = False
        Alarm_ON = True
    if (SysMin == AlarmMin + 3):                         #3 min past wake up time
        time.sleep(0.3)
        Buzzer.value = True
        time.sleep(0.3)
        Buzzer.value = False
        Alarm_ON = True
    if (SysMin == AlarmMin + 4):                         #4 min past wake up time
        time.sleep(0.5)
        Buzzer.value = True
        time.sleep(0.5)
        Buzzer.value = False
        Alarm_ON = True
    if (SysMin == AlarmMin + 5):                         #5 min past wake up time
        Buzzer.value = True
        time.sleep(1)
        Buzzer.value = False
        Alarm_ON = True


def Check_Problem_Time():
    global Stunde
    global Minute
    global Alarm_H
    global Alarm_M
    global Fan
    global Buzzer
    global Alarm_ON
    global num_leds
    global ProblemTime
    if ((Stunde == 23 and (Alarm_H ==23 or Alarm_H ==0)) or (Stunde == 0 and (Alarm_H== 0 or Alarm_H== 23))):
        #print("Übergangsalarm")
        if (Alarm_M > 55):
            if (Minute > 50 and SoftWake == True):
                AlarmLight()
                Fan.value = True
                Alarm_ON = True
            if (Minute > 55):
                AlarmLight()
                Fan.value = False
                Alarm_ON = True
                time.sleep(0.01)
                Buzzer.value = True
                time.sleep(0.01)
                Buzzer.value = False
            if (Minute < 6):
                AlarmLight()
                Fan.value = False
                Alarm_ON = True
                time.sleep(0.03)
                Buzzer.value = True
                time.sleep(0.02)
                Buzzer.value = False
        if (Alarm_M < 5 and Minute > 50 and SoftWake == True):
            AlarmLight()
            Fan.value = True
            Alarm_ON = True


def AlarmCancel():
    global Alarm_ON
    global Alarm_H
    global Alarm_M
    global Reset_H
    global Reset_M
    global Reset_Counter
    #print("CANCEL THE ALARM")
    Buzzer.value = True
    time.sleep(0.1)
    Buzzer.value = False
    strip.fill([0, 0, 0])
    strip.brightness = brightness_low
    if (Reset_Counter == 1):
        time.sleep(0.3)
        Buzzer.value = True
        time.sleep(0.5)
        Buzzer.value = False
        Alarm_H = Reset_H
        Alarm_M = Reset_M
        strip.fill([0, 0, 0])
        strip.write()
        Reset_H = 77
        Reset_M = 77
    else:
        Reset_H = Alarm_H
        Reset_M = Alarm_M
        Alarm_H = 77
        Alarm_M = 77
        Alarm_ON = False
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip.write()
    time.sleep(3)


def Demo():
    global num_leds
    strip.brightness = brightness_high
    strip.brightness = brightness_high
    for i in range(0, 100, 1):
        R = random.randint(0,255)
        G = R = random.randint(0,255)
        B = R = random.randint(0,255)
        strip.fill([R, G, B])
        time.sleep(0.2)
        strip.fill([0, 0, 0])

    for x in range(0, 10):
        for i in range(0, num_leds, 1):
            strip[(offset + i) % num_leds] = 255,0,0
            strip[(offset + (i+1)) % num_leds] = 255,0,0
            time.sleep(0.05)
            strip.fill([0, 0, 0])
        strip.write()  # and issue data to LED strip
        for i in range(0, num_leds, 1):
            strip[(offset + i) % num_leds] = 0,255,0
            strip[(offset + (i+1)) % num_leds] = 0,255,0
            time.sleep(0.05)
            strip.fill([0, 0, 0])
        strip.write()  # and issue data to LED strip
        for i in range(0, num_leds, 1):
            strip[(offset + i) % num_leds] = 0,0,255
            strip[(offset + (i+1)) % num_leds] = 0,05,255
            time.sleep(0.05)
            strip.fill([0, 0, 0])
        strip.write()  # and issue data to LED strip


def LightON():
    global num_leds
    global DIP5
    if (DIP5 == 1):
        strip.brightness = brightness_high
        strip.fill([0, 0, 0])
        x = random.randint(0,9)
        strip[(offset + x) % num_leds] = 255,255,255
        strip[(offset + x + 1) % num_leds] = 255,255,255
        strip[(offset + x + 2) % num_leds] = 255,255,255
        strip.write()
    DIP5 = DIP5 + 1
    if (DIP5 == 2000):
        DIP5 = 1





def debug():
    global Alarm_H
    global Alarm_M
    yr = 2023
    mt = 10
    dy = 28
    sc = 40
    wd = 1

    hr = 10
    mn = 10

    Alarm_H = 10
    Alarm_M = 11
    set_time = time.struct_time((yr, mt, dy, hr, mn, sc, wd, -1, -1)) # Year, Month, Date, Hour, Minutes, Seconds, Week Day
    rtc.datetime = set_time




#debug()
strip.fill([0, 0, 0])
strip.write()  # and issue data to LED strip


# =======       Main Programm       =======
if __name__ == "__main__":
  CLK = board.GP27
  DIO = board.GP26
  display = TM1637.TM1637(CLK, DIO)


if (btn_DIP3.value == False):
    #print("POWER FAIL ALARM")
    x = 0
    while (btn_Select.value == True):
        x = x + 1
        time.sleep(0.25)
        display.numbers(77,99, True)
        if (x < 240):
            Buzzer.value = True
        time.sleep(0.25)
        display.numbers(99,77, True)
        Buzzer.value = False

while True:
    t = rtc.datetime
    Minute = (t.tm_min)
    Stunde = (t.tm_hour)
    #print("Alarm_H",Alarm_H,"Alarm_M",Alarm_M," ResetCounter ",Reset_Counter)
    #print("Reset_H",Reset_H,"Reset_M",Reset_M)
    if (Alarm_H > 70):
        Reset_Counter = 1
    else:
        Reset_Counter = 0

    if (Alarm_H == 77 and Reset_H == 77):
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip[(offset + random.randint(1,12)) % num_leds] = 255,0,0
        strip.write()
        Alarm_H = 88


    if (btn_T.value == False):
        #print("KIPPSCHALTER DEMO")
        Demo()
    if (btn_DIP1.value == False):
        #print("DIP1 = Sommer Zeit")
        Stunde = Stunde + 1
    if (btn_DIP2.value == False):
        #print("DIP2 EIN SET NEW TIME IN RTC")
        SetTheTime()
    if (btn_DIP4.value == False):
        #print("SOFT WAKE OFF")
        SoftWake = False
    if (btn_DIP5.value == False):
        #print("DIP5 ON Swhitch Lights On")
        LightON()
    if(DIP5 > 0 and btn_DIP5.value == True):
        #print("Switch OFF Lights")
        strip.fill([0, 0, 0])
        DIP5 = 0
    if (Stunde == 24):
        Stunde = 0
    if (Stunde < 19):
        display.brightness(3)
    if (Stunde >= 19 or Stunde < 7):
        display.brightness(0)
    if(btn_AlarmOff.value == False):
        AlarmCancel()
    Check_Alarm()
    Check_Problem_Time()
    #print(f"The date is {days[int(t.tm_wday)]} {t.tm_mday}/{t.tm_mon}/{t.tm_year}")
    #print ("Sys  Time ", Stunde, ":", Minute)
    #print ("AlarmTime ",Alarm_H, ":", Alarm_M)
    if(Alarm_ON == False):
        Pause()  # wait a second
    #t = time.localtime()
    display.numbers(Stunde,Minute, False)
    if(Alarm_ON == False):
        Pause()
    display.numbers(Stunde,Minute, True)
    #time.sleep(60-(t.tm_sec%60))

